/*
 * PM - 2016
 * lab0.c
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#define DELAY_SECVENTA		250
#define NUM_SECVENTE		6

#define MORSE_LUNG		3
#define MORSE_SCURT		1

#define NUM_NOTE		8
#define DURATA_NOTA		100

float frecventa_nota[44] = {
	1.0, 32.70, 36.71, 41.20, 43.65, 49.0, 55.0, 61.74, 65.41, 73.42, 82.41, 87.31, 98.0, 110.0, 123.47, 130.81, 146.82, 164.81, 174.61, 196.0, 220.0, 246.84, 261.63, 293.66, 329.63, 349.23, 392.0, 440.0, 493.88, 523.25, 587.33, 659.25, 698.46 , 784.0, 880.0, 987.77, 1046.5, 1174.66, 1318.51, 1396.91, 1567.98, 1760.00, 1975.53, 1.0
};

int melody[100];


/* Genereaza o nota folosind buzzer-ul. */
void buzzer_reda(int nota) {
	float perioada_nota = 100000 / frecventa_nota[nota];
	int i, j;

	for (i = 0; i < DURATA_NOTA; ++i) {
		PORTD |= (1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);

		PORTD &= ~(1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0 ; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}
}

/* Aprinde LED-ul corespunzator notei alese. */
void led_aprinde(int nota) {
	PORTA = (1 << nota);
}

/* Stinge LED-urile corespunzatoare. */
void led_stinge(int rand) {
	if (rand == 0)
		PORTA = 0x00;
	else if (rand == 2)
		PORTC = 0x00;
	else if (rand == 3)
		PORTD &= ~((1 << PD0) | (1 << PD1));
}

/*
 * Genereaza note folosind buzzer-ul. Nota generata este aleasa folosind
 * butoanele PB0 - PB6, gama este schimbata folosiind butoanele PB7 si PD4, se poate inregistra folosind PD2 ce poate fi redata folosind PD3.
 * LED-urile de pe PC0 - PC5 indica gama curenta, iar LED-urile de pe PA0 - PA6 indica nota apasata.
 */
void piano(void) {
	int i = 0;

	int nota_curenta = -1;

	int gama = 1;

	int recording = 0;

	int playback = 0;

	int note_slot = 0;

	DDRA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6);

	PORTA = 0x00;

	DDRB = 0x00;

	PORTB = 0xFF;

	DDRC |= (1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3) | (1 << PC4) | (1 << PC5);

	PORTC = 0x00;

	DDRD |= (1 << PD0) | (1 << PD1) | (1 << PD5);

	DDRD &= ~((1 << PD2) | (1 << PD3) | (1 << PD4));

	PORTD &= ~((1 << PD0) | (1 << PD1) | (1 << PD5));

	PORTD |= (1 << PD2) | (1 << PD3) | (1 << PD4);

	while (1) {
		led_stinge(1);
		led_stinge(2);
		led_stinge(3);	

		PORTC |= (1 << (gama - 1));

		if ((PIND & (1 << PD2)) == 0) {
			if (note_slot == 0)
				recording = 1;
			else if (recording == 1)
				recording = 0;
			else if (playback == 1) {
				playback = 0;
				i = 0;
			}
			else
				note_slot = 0;
		}

		if ((PIND & (1 << PD3)) == 0)
			if (recording == 0)
				playback = 1;

		if (recording == 1) {
			if (note_slot == 100)
				recording = 0;
			else
				PORTD |= (1 << PD0);
		}
		
		if (playback == 1) {
			if (i == note_slot) {
				i = 0;
				playback = 0;
			}
			else {
				nota_curenta = melody[i];
				PORTD |= (1 << PD1);
				i++;
			}
		}
		else {

			if ((PINB & (1 << PB0)) == 0)
				nota_curenta = 1 * gama;

			if ((PINB & (1 << PB1)) == 0)
				nota_curenta = 2 * gama;
	
			if ((PINB & (1 << PB2)) == 0)
				nota_curenta = 3 * gama;

			if ((PINB & (1 << PB3)) == 0)
				nota_curenta = 4 * gama;

			if ((PINB & (1 << PB4)) == 0)
				nota_curenta = 5 * gama;

			if ((PINB & (1 << PB5)) == 0)
				nota_curenta = 6 * gama;

			if ((PINB & (1 << PB6)) == 0)
				nota_curenta = 7 * gama;

			if ((PINB & (1 << PB7)) == 0)
				if (gama < 6)
					gama++;

			if ((PIND & (1 << PD4)) == 0)
				if (gama > 1)
					gama--;

			if ((PIND & (1 << PD2)) == 0)
				PORTD |= (1 << PD0);
	
			if ((PIND & (1 << PD3)) == 0)
				PORTD |= (1 << PD1);
		}

		if (nota_curenta != -1) {
			led_aprinde(nota_curenta % 7);

			buzzer_reda(nota_curenta);
		}

		if (recording == 1) {
			melody[note_slot] = nota_curenta;
			note_slot++;
		}

		_delay_ms(50);
		nota_curenta = -1;
	}
}

int main(void) {
	piano();

	return 0;
}
